import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import React, { Component, createRef } from "react";

var LazyHydrate =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(LazyHydrate, _Component);

  function LazyHydrate(props) {
    var _this;

    _this = _Component.call(this, props) || this;
    _this.childRef = void 0;
    _this.cleanupFns = void 0;
    _this.io = void 0;
    _this.state = {
      // Always render on server
      hydrated: typeof window === "undefined"
    };
    var ssrOnly = props.ssrOnly,
        whenIdle = props.whenIdle,
        whenVisible = props.whenVisible;

    if (!ssrOnly && !whenIdle && !whenVisible) {
      console.warn("LazyHydrate: Set atleast one of the props to 'true'");
    }

    _this.childRef = createRef();
    _this.cleanupFns = [];
    _this.hydrate = _this.hydrate.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.io = typeof IntersectionObserver !== "undefined" ? new IntersectionObserver(function (entries) {
      entries.forEach(function (entry) {
        if (entry.target.parentElement === _this.childRef.current && (entry.isIntersecting || entry.intersectionRatio > 0)) {
          _this.hydrate();
        }
      });
    }) : null;
    return _this;
  }

  var _proto = LazyHydrate.prototype;

  _proto.hydrate = function hydrate() {
    this.setState({
      hydrated: true
    });

    while (this.cleanupFns.length > 0) {
      this.cleanupFns.pop()();
    }
  };

  _proto.componentDidMount = function componentDidMount() {
    var _this2 = this;

    if (this.childRef.current.childElementCount === 0) {
      // No SSR rendered content.
      this.hydrate();
      return;
    }

    var _this$props = this.props,
        ssrOnly = _this$props.ssrOnly,
        whenIdle = _this$props.whenIdle,
        whenVisible = _this$props.whenVisible;
    if (ssrOnly) return;

    if (whenIdle) {
      // @ts-ignore
      if (window.requestIdleCallback) {
        // @ts-ignore
        var id = window.requestIdleCallback(function () {
          return requestAnimationFrame(function () {
            return _this2.hydrate();
          });
        }, {
          timeout: 500
        }); // @ts-ignore

        this.cleanupFns.push[function () {
          return cancelIdleCallback(id);
        }];
      } else {
        this.hydrate();
      }
    }

    if (whenVisible) {
      if (this.io) {
        // As root node does not have any box model, it cannot intersect.
        this.io.observe(this.childRef.current.children[0]);
        this.cleanupFns.push(function () {
          return _this2.io.unobserve(_this2.childRef.current.children[0]);
        });
      } else {
        this.hydrate();
      }
    }
  };

  _proto.render = function render() {
    if (this.state.hydrated) {
      return React.createElement("div", {
        ref: this.childRef,
        style: {
          display: "contents"
        }
      }, this.props.children);
    } else {
      return React.createElement("div", {
        ref: this.childRef,
        style: {
          display: "contents"
        },
        dangerouslySetInnerHTML: {
          __html: ""
        }
      });
    }
  };

  return LazyHydrate;
}(Component);

export default LazyHydrate;